@@ -6,11 +6,11 @@ WWW::Mechanize - Handy web browsing in a Perl object
 
 =head1 VERSION
 
-Version 1.26
+Version 1.32
 
 =cut
 
-our $VERSION = '1.26';
+our $VERSION = '1.32';
 
 =head1 SYNOPSIS
 
@@ -57,7 +57,7 @@ traverse.
 
     $mech->back();
 
-If you want finer control over over your page fetching, you can use
+If you want finer control over your page fetching, you can use
 these methods. C<follow_link> and C<submit_form> are just high
 level wrappers around them.
 
@@ -81,6 +81,12 @@ FAQ in WWW::Mechanize::FAQ for more.
 
 =over 4
 
+=item * L<http://code.google.com/p/www-mechanize/issues/list>
+
+The queue for bugs & enhancements in WWW::Mechanize and
+Test::WWW::Mechanize.  Please note that the queue at L<http://rt.cpan.org>
+is no longer maintained.
+
 =item * L<http://search.cpan.org/dist/WWW-Mechanize/>
 
 The CPAN documentation page for Mechanize.
@@ -89,11 +95,6 @@ The CPAN documentation page for Mechanize.
 
 Frequently asked questions.  Make sure you read here FIRST.
 
-=item * L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=WWW-Mechanize>
-
-The RT queue for bugs & enhancements in Mechanize.  Click the "Report bug"
-link if your bug isn't already reported.
-
 =back
 
 =cut
@@ -177,7 +178,7 @@ calling C<< $mech->quiet(1) >>.  Default is off.
 
 =item * C<< stack_depth => $value >>
 
-Sets the depth of the page stack that keeps tracks of all the downloaded
+Sets the depth of the page stack that keeps track of all the downloaded
 pages. Default is 0 (infinite). If the stack is eating up your memory,
 then set it to 1.
 
@@ -198,7 +199,7 @@ sub new {
         onwarn      => \&WWW::Mechanize::_warn,
         onerror     => \&WWW::Mechanize::_die,
         quiet       => 0,
-        stack_depth => 867_5309,    # Arbitrarily humungous stack
+        stack_depth => 8675309,     # Arbitrarily humongous stack
         headers     => {},
     );
 
@@ -263,7 +264,23 @@ sets your User-Agent to
 
     Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1)
 
-The list of valid aliases can be returned from C<known_agent_aliases()>.
+The list of valid aliases can be returned from C<known_agent_aliases()>.  The current list is:
+
+=over
+
+=item * Windows IE 6
+
+=item * Windows Mozilla
+
+=item * Mac Safari
+
+=item * Mac Mozilla
+
+=item * Linux Mozilla
+
+=item * Linux Konqueror
+
+=back
 
 =cut
 
@@ -336,7 +353,7 @@ sub get {
     return $self->SUPER::get( $uri->as_string, @_ );
 }
 
-=head2 $mech->put( $uri, 'content' => $content )
+=head2 $mech->put( $uri, content => $content )
 
 PUTs I<$content> to $uri.  Returns an L<HTTP::Response> object.
 I<$uri> can be a well-formed URI string, a L<URI> object, or a
@@ -537,7 +554,7 @@ sub content {
         my %parms = @_;
         if ( exists $parms{base_href} ) {
             my $arg = (delete $parms{base_href}) || $self->base;
-            $content=~s/<head>/<head>\n<base href="$arg">/;
+            $content=~s/<head>/<head>\n<base href="$arg">/i;
         }
         if ( my $arg = delete $parms{format} ) {
             if ($arg eq 'text') {
@@ -678,6 +695,16 @@ in the page.
 
 Matches the name of the link against I<string> or I<regex>, as appropriate.
 
+=item * C<< id => string >> and C<< id_regex => regex >>
+
+Matches the attribute 'id' of the link against I<string> or
+I<regex>, as appropriate.
+
+=item * C<< class => string >> and C<< class_regex => regex >>
+
+Matches the attribute 'class' of the link against I<string> or
+I<regex>, as appropriate.
+
 =item * C<< tag => string >> and C<< tag_regex => regex >>
 
 Matches the tag that the link came from against I<string> or I<regex>,
@@ -728,7 +755,7 @@ sub find_link {
 
     my $wantall = ( $parms{n} eq 'all' );
 
-    $self->_clean_keys( \%parms, qr/^(n|(text|url|url_abs|name|tag)(_regex)?)$/ );
+    $self->_clean_keys( \%parms, qr/^(n|(text|url|url_abs|name|tag|id|class)(_regex)?)$/ );
 
     my @links = $self->links or return;
 
@@ -774,6 +801,11 @@ sub _match_any_link_parms {
     return if defined $p->{tag}           && !($link->tag && $link->tag eq $p->{tag} );
     return if defined $p->{tag_regex}     && !($link->tag && $link->tag =~ $p->{tag_regex} );
 
+    return if defined $p->{id}            && !($link->attrs->{id} && $link->attrs->{id} eq $p->{id} );
+    return if defined $p->{id_regex}      && !($link->attrs->{id} && $link->attrs->{id} =~ $p->{id_regex} );
+    return if defined $p->{class}         && !($link->attrs->{class} && $link->attrs->{class} eq $p->{class} );
+    return if defined $p->{class_regex}   && !($link->attrs->{class} && $link->attrs->{class} =~ $p->{class_regex} );
+
     # Success: everything that was defined passed.
     return 1;
 
@@ -838,7 +870,7 @@ sub find_all_links {
     return $self->find_link( @_, n=>'all' );
 }
 
-=head2 find_all_inputs( ... criteria ... )
+=head2 $mech->find_all_inputs( ... criteria ... )
 
 find_all_inputs() returns an array of all the input controls in the
 current form whose properties match all of the regexes passed in.
@@ -935,7 +967,7 @@ Finds an image in the current page. It returns a
 L<WWW::Mechanize::Image> object which describes the image.  If it fails
 to find an image it returns undef.
 
-You can select which link to find by passing in one or more of these
+You can select which image to find by passing in one or more of these
 key/value pairs:
 
 =over 4
@@ -1089,7 +1121,7 @@ calls to C<L<field()>> and C<L<click()>>.  Also returns the form that was
 selected.
 
 If it is found, the form is returned as an L<HTML::Form> object and set internally
-for later used with Mech's form methods such as C<L<field()>> and C<L<click()>>.
+for later use with Mech's form methods such as C<L<field()>> and C<L<click()>>.
 
 Emits a warning and returns undef if no form is found.
 
@@ -1118,7 +1150,7 @@ with that name, then the first one is used, and a warning is
 generated.
 
 If it is found, the form is returned as an L<HTML::Form> object and set internally
-for later used with Mech's form methods such as C<L<field()>> and C<L<click()>>.
+for later use with Mech's form methods such as C<L<field()>> and C<L<click()>>.
 
 Returns undef if no form is found.
 
@@ -1615,10 +1647,15 @@ are a list of key/value pairs, all of which are optional.
 
 =item * fields => \%fields
 
+Specifies the fields to be filled in the current form.
+
+=item * with_fields => \%fields
+
 Probably all you need for the common case. It combines a smart form selector
 and data setting in one operation. It selects the first form that contains all
 fields mentioned in C<\%fields>.  This is nice because you don't need to know
 the name or number of the form to do this.
+
 (calls C<L<form_with_fields>> and C<L<set_fields()>>).
 
 If you choose this, the form_number, form_name and fields options will be ignored.
@@ -1817,8 +1854,93 @@ sub save_content {
     open( my $fh, '>', $filename ) or $self->die( "Unable to create $filename: $!" );
     print {$fh} $self->content or $self->die( "Unable to write to $filename: $!" );
     close $fh or $self->die( "Unable to close $filename: $!" );
+
+    return;
 }
 
+=head2 $mech->dump_links( [[$fh], $absolute] )
+
+Prints a dump of the links on the current page to I<$fh>.  If I<$fh>
+is not specified or is undef, it dumps to STDOUT.
+
+If I<$absolute> is true, links displayed are absolute, not relative.
+
+=cut
+
+sub dump_links {
+    my $self = shift;
+    my $fh = shift || \*STDOUT;
+    my $absolute = shift;
+
+    for my $link ( $self->links ) {
+        my $url = $absolute ? $link->url_abs : $link->url;
+        $url = '' if not defined $url;
+        print {$fh} $url, "\n";
+    }
+    return;
+}
+
+=head2 $mech->dump_images( [[$fh], $absolute] )
+
+Prints a dump of the images on the current page to I<$fh>.  If I<$fh>
+is not specified or is undef, it dumps to STDOUT.
+
+If I<$absolute> is true, links displayed are absolute, not relative.
+
+=cut
+
+sub dump_images {
+    my $self = shift;
+    my $fh = shift || \*STDOUT;
+    my $absolute = shift;
+
+    for my $image ( $self->images ) {
+        my $url = $absolute ? $image->url_abs : $image->url;
+        $url = '' if not defined $url;
+        print {$fh} $url, "\n";
+    }
+    return;
+}
+
+=head2 $mech->dump_forms( [$fh] )
+
+Prints a dump of the forms on the current page to I<$fh>.  If I<$fh>
+is not specified or is undef, it dumps to STDOUT.
+
+=cut
+
+sub dump_forms {
+    my $self = shift;
+    my $fh = shift || \*STDOUT;
+
+    for my $form ( $self->forms ) {
+        print {$fh} $form->dump, "\n";
+    }
+    return;
+}
+
+=head2 $mech->dump_all( [[$fh], $absolute] )
+
+Prints a dump of all links, images and forms on the current page to
+I<$fh>.  If I<$fh> is not specified or is undef, it dumps to STDOUT.
+
+If I<$absolute> is true, links displayed are absolute, not relative.
+
+=cut
+
+sub dump_all {
+    my $self = shift;
+    my $fh = shift || \*STDOUT;
+    my $absolute = shift;
+
+    $self->dump_links( $fh, $absolute );
+    $self->dump_images( $fh, $absolute );
+    $self->dump_forms( $fh, $absolute );
+
+    return;
+}
+
+
 =head1 OVERRIDDEN LWP::UserAgent METHODS
 
 =head2 $mech->clone()
@@ -1881,6 +2003,8 @@ sub request {
     }
 
     $self->_update_page($request, $self->_make_request( $request, @_ ));
+
+    # XXX This should definitively return something.
 }
 
 =head2 $mech->update_html( $html )
@@ -2016,7 +2140,9 @@ sub _update_page {
 
     # Try to decode the content. Undef will be returned if there's nothing to decompress.
     # See docs in HTTP::Message for details. Do we need to expose the options there? 
-    my $content = $res->decoded_content;
+    # use charset => 'none' because while we want LWP to handle Content-Encoding for 
+    # the auto-gzipping with Compress::Zlib we don't want it messing with charset
+    my $content = $res->decoded_content( charset => 'none' );
     $content = $res->content if (not defined $content);
 
     $content .= _taintedness();
@@ -2032,18 +2158,48 @@ sub _update_page {
 } # _update_page
 
 our $_taintbrush;
+
+# This is lifted wholesale from Test::Taint
 sub _taintedness {
-    if ( not defined $_taintbrush ) {
-        my $file = $0;
-        open( my $fh, '<', $file ) or die "Can't open $file: $!";
-        sysread( $fh, $_taintbrush, 1 ) or die "Can't read from $file: $!";
-        $_taintbrush = substr( $_taintbrush, 0, 0 );
-        close $fh;
+    return $_taintbrush if defined $_taintbrush;
+
+    # Somehow we need to get some taintedness into our $_taintbrush.
+    # Let's try the easy way first. Either of these should be
+    # tainted, unless somebody has untainted them, so this
+    # will almost always work on the first try.
+    # (Unless, of course, taint checking has been turned off!)
+    $_taintbrush = substr("$0$^X", 0, 0);
+    return $_taintbrush if _is_tainted( $_taintbrush );
+
+    # Let's try again. Maybe somebody cleaned those.
+    $_taintbrush = substr(join("", @ARGV, %ENV), 0, 0);
+    return $_taintbrush if _is_tainted( $_taintbrush );
+
+    # If those don't work, go try to open some file from some unsafe
+    # source and get data from them.  That data is tainted.
+    # (Yes, even reading from /dev/null works!)
+    for my $filename ( qw(/dev/null / . ..), values %INC, $0, $^X ) {
+        if ( open my $fh, '<', $filename ) {
+            my $data;
+            if ( defined sysread $fh, $data, 1 ) {
+                $_taintbrush = substr( $data, 0, 0 );
+                last if _is_tainted( $_taintbrush );
+            }
+        }
     }
 
+    # Sanity check
+    die "Our taintbrush should have zero length!" if length $_taintbrush;
+
     return $_taintbrush;
 }
 
+sub _is_tainted {
+    no warnings qw(void uninitialized);
+
+    return !eval { join('', shift), kill 0; 1 };
+} # _is_tainted
+
 
 =head2 $mech->_modify_request( $req )
 
@@ -2359,9 +2515,9 @@ __END__
 
 =head1 WWW::MECHANIZE'S SUBVERSION REPOSITORY
 
-Mech is hosted by the kind generosity of Ask and Robert,
-maintainers of perl.org.  The Subversion repository is at
-L<http://svn.perl.org/modules/www-mechanize>.
+Mech and Test::WWW::Mechanize are both hosted at Google Code:
+http://code.google.com/p/www-mechanize/.  The Subversion repository
+is at http://www-mechanize.googlecode.com/svn/wm/.
 
 =head1 OTHER DOCUMENTATION
 
@@ -2510,8 +2666,8 @@ Just like Mech, but using Microsoft Internet Explorer to do the work.
 
 Please report any requests, suggestions or (gasp!) bugs via the
 excellent RT bug-tracking system at http://rt.cpan.org/, or email to
-bug-WWW-Mechanize@rt.cpan.org.  This makes it much easier for me to
-track things.
+C<bug-WWW-Mechanize at rt.cpan.org>.  This makes it much easier for
+me to track things.
 
 L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=WWW-Mechanize> is the RT queue
 for Mechanize.  Please check to see if your bug has already been reported.
@@ -2523,7 +2679,10 @@ to read the FAQ if you have support requests.
 
 Thanks to the numerous people who have helped out on WWW::Mechanize in
 one way or another, including
-Kirrily Robert for the orignal C<WWW::Automate>,
+Kirrily Robert for the original C<WWW::Automate>,
+Adriano Ferreira,
+Miyagawa,
+Peteris Krumins,
 Rafael Kitover,
 David Steinbrunner,
 Kevin Falcone,
@@ -2557,6 +2716,7 @@ Julien Beasley,
 Abe Timmerman,
 Peter Stevens,
 Pete Krawczyk,
+Tad McClellan,
 and the late great Iain Truskett.
 
 =head1 COPYRIGHT
